//
//  ButtonViewController.swift
//  Button
//
//  Created by Orkhan Alikhanov on 10/9/18.
//  Copyright © 2018 CosmicMind. All rights reserved.
//

import Material

class ButtonViewController: AppViewController {
  struct ButtonLayout {
    struct Flat {
      static let width: CGFloat = 120
      static let height: CGFloat = 44
      static let offsetY: CGFloat = -150
    }
    
    struct Raised {
      static let width: CGFloat = 150
      static let height: CGFloat = 44
      static let offsetY: CGFloat = -75
    }
    
    struct Fab {
      static let diameter: CGFloat = 48
    }
    
    struct Icon {
      static let width: CGFloat = 120
      static let height: CGFloat = 48
      static let offsetY: CGFloat = 75
    }
  }
  
  
  override func prepare() {
    super.prepare()
    title = title ?? "Button"
    
    let flatButton = FlatButton(title: "Flat Button")
    view.layout(flatButton)
      .width(ButtonLayout.Flat.width)
      .height(ButtonLayout.Flat.height)
      .center(offsetY: ButtonLayout.Flat.offsetY)
    
    
    let raisedButton = RaisedButton(title: "Show dialog")
    view.layout(raisedButton)
      .width(ButtonLayout.Raised.width)
      .height(ButtonLayout.Raised.height)
      .center(offsetY: ButtonLayout.Raised.offsetY)
    
    
    let fabButton = FABButton(image: Icon.cm.add)
    view.layout(fabButton)
      .width(ButtonLayout.Fab.diameter)
      .height(ButtonLayout.Fab.diameter)
      .center()
    
    
    let iconButton = IconButton(image: Icon.cm.search)
    view.layout(iconButton)
      .width(ButtonLayout.Icon.width)
      .height(ButtonLayout.Icon.height)
      .center(offsetY: ButtonLayout.Icon.offsetY)
    
    raisedButton.addTarget(self, action: #selector(didTapShowDialog), for: .touchUpInside)
  }
  
  @objc
  func didTapShowDialog() {
    showDialog()
//    showCustomDialog()
  }
  
  func showDialog() {
    Dialog()
      .title("Use Google's location service?")
      .details("Let Google help apps determine location. This means sending anonymous location data to Google, even when no apps are running. Let Google help apps determine location. This means sending anonymous location data to Google, even when no apps are running. Let Google help apps determine location. This means sending anonymous location data to Google, even when no apps are running. Let Google help apps determine location. This means sending anonymous location data to Google, even when no apps are running. Let Google help apps determine location. This means sending anonymous location data to Google, even when no apps are running. Let Google help apps determine location. This means sending anonymous location data to Google, even when no apps are running.")
      .delegate(self)
      .isCancelable(true) {
        print("cancelled")
      }
      .willAppear {
        print("will appear")
      }
      .didDisappear {
        print("did disappear")
      }
      .positive("AGREE") {
        print("User agreed")
      }
      .negative("DISAGREE") {
        print("User disagreed")
      }
      .neutral("CANCEL") {
        print("did tap cancel")
      }
      //      .shouldDismiss { _, _ in false }
      .show(self)
  }
  
  func showCustomDialog() {
    
    class RadioDialogView: DialogView {
      let buttonGroup = RadioButtonGroup()
      
      override func prepare() {
        super.prepare()
        buttonGroup.backgroundColor = nil
        contentArea.layout(buttonGroup).edges()
      }
      
      override func contentAreaSizeThatFits(width: CGFloat) -> CGSize {
        var s = buttonGroup.sizeThatFits(CGSize(width: width, height: .greatestFiniteMagnitude))
        s.width = max(s.width, Screen.width * 0.8)
        return s
      }
    }
    
    let v = DialogController<RadioDialogView>()
    v.dialogView.titleLabel.text = "Radio dialog"
    v.dialogView.positiveButton.title = "OK"
    v.dialogView.negativeButton.title = "CANCEL"
    v.dialogView.buttonGroup.buttons = (1...10).map { RadioButton(title: "Option \($0)") }
    
    present(v, animated: true, completion: nil)
  }
}

extension ButtonViewController: DialogDelegate {
  func dialog(_ dialog: Dialog, didTapPositive button: Button) {
    print(#function)
  }
  
  func dialogDidCancel(_ dialog: Dialog) {
    print(#function)
  }
  
  func dialogDidDisappear(_ dialog: Dialog) {
    print(#function)
  }
  
  func dialogWillAppear(_ dialog: Dialog) {
    print(#function)
  }
  
  func dialog(_ dialog: Dialog, didTapNeutral button: Button) {
    print(#function)
  }
  
  func dialog(_ dialog: Dialog, didTapNegative button: Button) {
    print(#function)
  }
  
  func dialog(_ dialog: Dialog, shouldDismiss button: Button?) -> Bool {
    print(#function)
    return true // button === dialog.controller.dialogView.neutralButton
  }
}
